package db.util;

import lombok.extern.slf4j.Slf4j;

import java.io.*;

@Slf4j
public class FileUtils {
    private static FileUtils instance = new FileUtils();

    public static FileUtils getInstance() {
        return instance;
    }

    public static FileOutputStream openFile(String fileName) {
        File file = new File(fileName);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }
        FileOutputStream writer = null;
        try {
            writer = new FileOutputStream(file, true);
        } catch (FileNotFoundException e) {
            log.error("无法打开文件{}", fileName, e);
        }
        return writer;

    }

    public static void deleteFile(String filename) {
        File file = new File(filename);
        if (file.exists()) {
            file.delete();
        }
    }

    public static void appendFile(String filename, byte[] content) throws IOException {
        File file = new File(filename);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }

        // 追加写入
        FileOutputStream writer = new FileOutputStream(file, true);
        writer.write(content);
        writer.close();
    }

    public static void overwriteFile(String filename, byte[] content, int pos) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(filename, "rw");
        randomAccessFile.seek(pos);
        randomAccessFile.write(content);
        randomAccessFile.close();
    }

    public static void readFile(String filename, byte[] content, int pos) throws IOException {
        RandomAccessFile randomAccessFile = new RandomAccessFile(filename, "r");
        randomAccessFile.seek(pos);
        randomAccessFile.readFully(content);
        randomAccessFile.close();
    }

    public static int fileLength(String filename) {
        File file = new File(filename);
        if (file.exists() && file.isFile()) {
            return (int) file.length();
        }
        return 0;
    }


    public static void writeFile(String filename, byte[] content) throws IOException {
        File file = new File(filename);
        // 创建文件夹
        File directory = file.getParentFile();
        if (directory != null) {
            directory.mkdirs();
        }

        // 追加写入
        FileOutputStream writer = new FileOutputStream(file, false);
        writer.write(content);
        writer.close();
    }

}
