package db.entity;

import db.Globals;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;

import java.util.Arrays;

@Data
public class LogFileMeta {
    private int magic = Globals.LOG_FILE_MAGIC;
    private Integer id;
    private int[] indices;
    private long minTime;
    private long maxTime;
    private String fileName;
    private Integer count;
    private Integer valid;
    private Integer saveDisk;
    private Integer saveTsdb;
    private int bodyLength;
    private int headerExtLength;
    private int headerTotalLength;

    public LogFileMeta(Integer id, String fileName) {
        this.id = id;
        this.fileName = fileName;
        this.indices = new int[Globals.MAX_LOG_ITEM];
        Arrays.fill(this.indices, -1);
        this.minTime = 0;
        this.maxTime = 0;
        this.count = 0;
        this.bodyLength = 0;
        this.headerExtLength = 0;
        this.valid = 1;
        this.saveDisk = 0;
        this.saveTsdb = 0;
        this.headerTotalLength = Globals.MAX_LOG_ITEM * 4 + 28 + headerExtLength;
    }

    public byte[] getPayload() {
        ByteBuf buf = Unpooled.buffer(headerTotalLength);
        buf.writeInt(magic);
        for (int i = 0; i < Globals.MAX_LOG_ITEM; i++) {
            buf.writeInt(indices[i]);
        }
        buf.writeLong(minTime);
        buf.writeLong(maxTime);
        buf.writeInt(bodyLength);
        buf.writeInt(headerExtLength);
        return buf.array();
    }

    public boolean setPayload(byte[] buffer) {
        ByteBuf buf = Unpooled.buffer(buffer.length);
        buf.writeBytes(buffer);

        magic = buf.readInt();
        if(magic != Globals.LOG_FILE_MAGIC) {
            return false;
        }

        for (int i = 0; i < Globals.MAX_LOG_ITEM; i++) {
            indices[i] = buf.readInt();
        }

        minTime = buf.readLong();
        maxTime = buf.readLong();
        bodyLength = buf.readInt();
        headerExtLength = buf.readInt();
        return true;
    }
}
