package db.entity;

import db.Globals;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Data
@Slf4j
@NoArgsConstructor
public class WalItem {
    int magic = Globals.WAL_ITEM_MAGIC;
    short probeId;
    byte port;
    byte dataStrategy;
    byte clockSync;
    short length;
    long stamp;
    byte[] rawPacket;

    public WalItem(short probeId, byte port, byte dataStrategy, byte clockSync, long stamp, byte[] rawPacket) {
        this.probeId = probeId;
        this.port = port;
        this.dataStrategy = dataStrategy;
        this.clockSync = clockSync;
        this.length = (short) (rawPacket.length + 20);
        this.stamp = stamp;
        this.rawPacket = rawPacket;
    }

    public byte[] getPayload() {
        ByteBuf buf = Unpooled.buffer(length);
        buf.writeInt(magic);
        buf.writeShort(probeId);
        buf.writeByte(0);
        buf.writeByte(port);
        buf.writeByte(dataStrategy);
        buf.writeByte(clockSync);
        buf.writeShort(length);
        buf.writeLong(stamp);
        buf.writeBytes(rawPacket);
        return buf.array();
    }

    public boolean setPayload(byte[] buffer) {
        ByteBuf buf = Unpooled.buffer(buffer.length);
        buf.writeBytes(buffer);

        magic = buf.readInt();
        if (magic != Globals.WAL_ITEM_MAGIC) {
            return false;
        }
        probeId = buf.readShort();
        buf.readByte(); // reserved
        port = buf.readByte();
        dataStrategy = buf.readByte();
        clockSync = buf.readByte();
        length = buf.readShort();
        stamp = buf.readLong();
        rawPacket = new byte[length - 20];
        buf.readBytes(rawPacket);
        return true;
    }

    /**
     * 从指定位置读取，存入到header部分
     *
     * @param buffer
     * @param startOffset
     * @return 是否成功
     */
    public boolean setHeader(byte[] buffer, int startOffset) {
        ByteBuf buf = Unpooled.buffer(20);
        buf.writeBytes(buffer, startOffset, 20);
        magic = buf.readInt();
        if (magic != Globals.WAL_ITEM_MAGIC) {
            return false;
        }
        probeId = buf.readShort();
        buf.readByte(); // reserved
        port = buf.readByte();
        dataStrategy = buf.readByte();
        clockSync = buf.readByte();
        length = buf.readShort();
        stamp = buf.readLong();
        return true;
    }

    /**
     * 填充this的body
     *
     * @param buffer
     * @param startOffset wal的startOffset
     * @param length      wal的length字段
     */
    public void setBody(byte[] buffer, int startOffset, int length) {
        rawPacket = new byte[length - 20];
        ConvertUtils.arrayCopy(buffer, startOffset + 20, rawPacket, 0, length - 20);
    }

    /**
     * 读取byte[]，填充this对象并返回读取长度
     *
     * @param buffer      待读取字节数组
     * @param startOffset 起始索引
     * @return 读取长度，为-1时表示读取失败
     */
    public int setContent(byte[] buffer, int startOffset) {

        boolean isReadSuccess = setHeader(buffer, startOffset);
        if (!isReadSuccess) {
            return -1;
        }

        setBody(buffer, startOffset, this.length);
        return this.length;
    }
}
