package db.mapper;

import db.entity.LogFileMeta;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

@Mapper
public interface LogFileMetaMapper {

    @Insert("insert into log_file_meta(id,file_name,min_time,max_time,count,valid,save_disk, save_tsdb)values(#{id}," +
            "#{fileName},#{minTime},#{maxTime},#{count},#{valid},#{saveDisk},#{saveTsdb})")
    public int insert(LogFileMeta logFileMeta);

    @Update("update log_file_meta set file_name = #{fileName}, min_time = #{minTime}, max_time = #{maxTime}," +
            "count = #{count}, valid = #{valid}, save_disk = #{saveDisk}, save_tsdb = #{saveTsdb} where id = #{id}")
    public int update(LogFileMeta logFileMeta);

    @Select("select * from log_file_meta")
    public List<LogFileMeta> getLogFileMeta();

    /**
     * 获取时间范围内所有（包括无效）的log文件
     * @param start
     * @param end
     * @return
     */
    @Select("select * from log_file_meta where not (min_time > #{arg1} or max_time < #{arg0})")
    public List<LogFileMeta> getLogFileMetaBetweenTime(long start, long end);

    /**
     * 获取时间范围内有效的log文件
     * @param start
     * @param end
     * @return
     */
    @Select("select * from log_file_meta where not (min_time > #{arg1} or max_time < #{arg0}) and valid=1")
    public List<LogFileMeta> getValidLogFileMetaBetweenTime(long start, long end);

    @Select("select * from log_file_meta order by id desc limit 1")
    public LogFileMeta getLastLogFileMeta();

    @Select("select * from log_file_meta where id = #{id}")
    public LogFileMeta getLogFileMetaById(long id);

    /**
     * 检查过期文件，仅查找有效文件
     * @param expireTime
     * @return
     */
    @Select("select * from log_file_meta where max_time < #{expireTime} and valid = 1")
    public List<LogFileMeta> getValidExpiredLogFileMeta(long expireTime);

    /**
     * 标记过期文件无效
     * @param expireTime
     * @return
     */
    @Select("update log_file_meta set valid = 0 where max_time < #{expireTime} and valid = 1")
    public List<LogFileMeta> markValidExpiredLogFileMetaAsInvalid(long expireTime);

}
