package db.service;

import db.Globals;
import db.entity.LogFileMeta;
import db.entity.LogItem;
import db.entity.WalItem;
import db.mapper.LogFileMetaMapper;
import db.util.ConvertUtils;
import db.util.FileUtils;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
@Data
@Slf4j
public class WalFileService {

    @Value("${system.wal-files.prefix}")
    private String prefix;
    @Value("${system.wal-files.suffix}")
    private String suffix;

    private String fileName;

    // 追加写入
    FileOutputStream walWriter;

    @PostConstruct
    public void openWalFile() {
        fileName = prefix + "0" + suffix;
        walWriter = FileUtils.openFile(fileName);
        log.info("打开wal文件：{}", fileName);
    }

    @PreDestroy
    public void closeWalFile() {
        try {
            walWriter.close();
            log.info("关闭wal文件：{}", fileName);
        } catch (IOException e) {
            log.error("无法关闭wal文件：{}", fileName, e);
        }
    }

    public void appendWal(WalItem walItem) throws IOException {
        // 写入数据
        byte[] walItemPayload = walItem.getPayload();


        if(walWriter == null) {
            openWalFile();
        }
        walWriter.write(walItemPayload);

        log.debug("将{}写入wal文件{}", walItem, fileName);
    }

    public List<WalItem> readWal() {
        int fileLength = FileUtils.fileLength(fileName);
        if (fileLength == 0) {
            return null;
        }
        byte[] buffer = new byte[fileLength];
        try {
            FileUtils.readFile(fileName, buffer, 0);
        } catch (IOException e) {
            log.error("读取wal文件失败，filename={}", fileName, e);
        }
        List<WalItem> walItems = new ArrayList<>();

        int offset = 0;
        while (offset < fileLength) {
            WalItem walItem = new WalItem();
            int readLength = walItem.setContent(buffer, offset);
            if (readLength == -1) {
                log.error("walItem文件完整性校验失败，文件名：{}，文件偏移量：{}", fileName, offset);
                return null;
            }
            offset += readLength;
            walItems.add(walItem);
        }
        return walItems;
    }

    public void clear() {
        closeWalFile();
        FileUtils.deleteFile(fileName);
        log.info("清空wal文件{}", fileName);
        openWalFile();
    }

}
